#!/usr/bin/env python3
"""
Email Security Analyzer
Advanced email security assessment and configuration analyzer.
Parameters:
    - domain: Target domain to analyze
    - mode: Analysis mode (basic, full)
    - output: Output file for results
Requires: dnspython, requests packages
"""
import dns.resolver
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
from collections import defaultdict
import socket
import requests
import concurrent.futures
import time
import re
import ssl
import smtplib
import email.utils
from email.mime.text import MIMEText

class EmailSecurityAnalyzer:
    def __init__(self, domain, mode='full'):
        self.domain = domain
        self.mode = mode.lower()
        self.results = defaultdict(dict)
        self.setup_logging()
        self.initialize_resolvers()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def initialize_resolvers(self):
        """Initialize DNS resolvers"""
        self.resolver = dns.resolver.Resolver()
        self.resolver.timeout = 5
        self.resolver.lifetime = 5

    def query_record(self, record_type, domain=None):
        """Query specific DNS record type"""
        try:
            answers = self.resolver.resolve(
                domain or self.domain,
                record_type
            )
            return [str(rdata) for rdata in answers]
        except Exception:
            return []

    def analyze_spf(self):
        """Analyze SPF record configuration"""
        txt_records = self.query_record('TXT')
        spf_records = [r for r in txt_records if r.startswith('v=spf1')]
        
        if not spf_records:
            self.results['spf'] = {
                'present': False,
                'valid': False,
                'record': None,
                'analysis': {
                    'score': 0,
                    'issues': ['No SPF record found']
                }
            }
            return

        spf_record = spf_records[0]
        analysis = {
            'score': 70,
            'mechanisms': [],
            'includes': [],
            'ip_ranges': [],
            'all_mechanism': None,
            'issues': []
        }

        # Parse SPF record
        mechanisms = spf_record.split()
        lookup_count = 0

        for mechanism in mechanisms:
            if mechanism.startswith('include:'):
                analysis['includes'].append(mechanism[8:])
                lookup_count += 1
            elif mechanism.startswith('ip4:') or mechanism.startswith('ip6:'):
                analysis['ip_ranges'].append(mechanism)
            elif mechanism.startswith('mx'):
                analysis['mechanisms'].append('mx')
                lookup_count += 1
            elif mechanism.startswith('a'):
                analysis['mechanisms'].append('a')
                lookup_count += 1
            elif mechanism in ['-all', '~all', '?all', '+all']:
                analysis['all_mechanism'] = mechanism

        # Evaluate configuration
        if lookup_count > 10:
            analysis['score'] -= 20
            analysis['issues'].append(
                'Too many DNS lookups (max 10 recommended)'
            )

        if not analysis['all_mechanism']:
            analysis['score'] -= 30
            analysis['issues'].append('Missing all mechanism')
        elif analysis['all_mechanism'] == '+all':
            analysis['score'] -= 40
            analysis['issues'].append(
                'Using +all is too permissive'
            )
        elif analysis['all_mechanism'] == '?all':
            analysis['score'] -= 20
            analysis['issues'].append(
                'Using ?all provides no protection'
            )

        if len(spf_records) > 1:
            analysis['score'] -= 20
            analysis['issues'].append(
                'Multiple SPF records found (only one allowed)'
            )

        self.results['spf'] = {
            'present': True,
            'valid': True,
            'record': spf_record,
            'analysis': analysis
        }

    def analyze_dmarc(self):
        """Analyze DMARC record configuration"""
        try:
            dmarc_records = self.query_record('TXT', f'_dmarc.{self.domain}')
            dmarc_record = next(
                (r for r in dmarc_records if r.startswith('v=DMARC1')),
                None
            )
        except:
            dmarc_record = None

        if not dmarc_record:
            self.results['dmarc'] = {
                'present': False,
                'valid': False,
                'record': None,
                'analysis': {
                    'score': 0,
                    'issues': ['No DMARC record found']
                }
            }
            return

        analysis = {
            'score': 70,
            'policy': None,
            'subdomain_policy': None,
            'aggregate_reports': [],
            'forensic_reports': [],
            'pct': 100,
            'issues': []
        }

        # Parse DMARC record
        parts = dmarc_record.split(';')
        for part in parts:
            part = part.strip()
            if '=' in part:
                key, value = part.split('=', 1)
                key = key.strip()
                value = value.strip()

                if key == 'p':
                    analysis['policy'] = value
                elif key == 'sp':
                    analysis['subdomain_policy'] = value
                elif key == 'rua':
                    analysis['aggregate_reports'].extend(
                        [u.strip() for u in value.split(',')]
                    )
                elif key == 'ruf':
                    analysis['forensic_reports'].extend(
                        [u.strip() for u in value.split(',')]
                    )
                elif key == 'pct':
                    try:
                        analysis['pct'] = int(value)
                    except:
                        pass

        # Evaluate configuration
        if not analysis['policy']:
            analysis['score'] -= 30
            analysis['issues'].append('Missing required policy (p) tag')
        elif analysis['policy'] == 'none':
            analysis['score'] -= 30
            analysis['issues'].append(
                'Policy is set to none, providing no protection'
            )
        elif analysis['policy'] == 'quarantine':
            analysis['score'] -= 10
            analysis['issues'].append(
                'Consider upgrading policy to reject for better protection'
            )

        if not analysis['subdomain_policy']:
            analysis['subdomain_policy'] = analysis['policy']
        elif analysis['subdomain_policy'] == 'none':
            analysis['score'] -= 20
            analysis['issues'].append(
                'Subdomain policy is none, leaving subdomains unprotected'
            )

        if not analysis['aggregate_reports']:
            analysis['score'] -= 20
            analysis['issues'].append('No aggregate report destinations specified')

        if analysis['pct'] < 100:
            analysis['score'] -= (100 - analysis['pct']) / 5
            analysis['issues'].append(
                f'Only applying policy to {analysis["pct"]}% of messages'
            )

        self.results['dmarc'] = {
            'present': True,
            'valid': True,
            'record': dmarc_record,
            'analysis': analysis
        }

    def analyze_mx(self):
        """Analyze MX record configuration"""
        mx_records = self.query_record('MX')
        
        if not mx_records:
            self.results['mx'] = {
                'present': False,
                'records': [],
                'analysis': {
                    'score': 0,
                    'issues': ['No MX records found']
                }
            }
            return

        analysis = {
            'score': 70,
            'issues': []
        }

        # Parse and validate MX records
        parsed_records = []
        for record in mx_records:
            try:
                priority, server = record.split(' ', 1)
                parsed_records.append({
                    'priority': int(priority),
                    'server': server.rstrip('.')
                })
            except:
                analysis['score'] -= 10
                analysis['issues'].append(
                    f'Invalid MX record format: {record}'
                )

        # Sort by priority
        parsed_records.sort(key=lambda x: x['priority'])

        # Check configuration
        if len(parsed_records) < 2:
            analysis['score'] -= 20
            analysis['issues'].append(
                'Only one MX record found - no backup mail server'
            )

        # Check for duplicate priorities
        priorities = [r['priority'] for r in parsed_records]
        if len(priorities) != len(set(priorities)):
            analysis['score'] -= 10
            analysis['issues'].append(
                'Duplicate MX record priorities found'
            )

        # Analyze each mail server
        for record in parsed_records:
            server = record['server']
            
            # Check if server resolves
            try:
                ip = socket.gethostbyname(server)
            except:
                analysis['score'] -= 20
                analysis['issues'].append(
                    f'MX server {server} does not resolve'
                )
                continue

            # Check SSL/TLS configuration
            try:
                context = ssl.create_default_context()
                with socket.create_connection((server, 25), timeout=5) as sock:
                    with context.wrap_socket(
                        sock, server_hostname=server
                    ) as ssock:
                        cipher = ssock.cipher()
                        record['ssl'] = {
                            'version': ssock.version(),
                            'cipher': cipher[0] if cipher else None
                        }
            except:
                analysis['score'] -= 15
                analysis['issues'].append(
                    f'Could not establish SSL/TLS connection to {server}'
                )
                record['ssl'] = None

        self.results['mx'] = {
            'present': True,
            'records': parsed_records,
            'analysis': analysis
        }

    def analyze_smtp_security(self):
        """Analyze SMTP server security configuration"""
        if 'mx' not in self.results or not self.results['mx']['records']:
            return

        smtp_results = []
        for record in self.results['mx']['records']:
            server = record['server']
            result = {
                'server': server,
                'starttls': False,
                'tls_version': None,
                'ciphers': [],
                'issues': []
            }

            try:
                # Test STARTTLS
                with smtplib.SMTP(server, timeout=5) as smtp:
                    smtp.ehlo()
                    if smtp.has_extn('starttls'):
                        result['starttls'] = True
                        try:
                            smtp.starttls()
                            result['tls_version'] = smtp.sock.version()
                            result['ciphers'].append(smtp.sock.cipher()[0])
                        except:
                            result['issues'].append(
                                'STARTTLS available but failed'
                            )
                    else:
                        result['issues'].append('STARTTLS not supported')

            except Exception as e:
                result['issues'].append(f'Connection failed: {str(e)}')

            smtp_results.append(result)

        self.results['smtp'] = {
            'servers': smtp_results,
            'analysis': self.analyze_smtp_results(smtp_results)
        }

    def analyze_smtp_results(self, results):
        """Analyze SMTP security test results"""
        analysis = {
            'score': 70,
            'issues': []
        }

        for result in results:
            if not result['starttls']:
                analysis['score'] -= 30
                analysis['issues'].append(
                    f'Server {result["server"]} does not support STARTTLS'
                )
            elif result['tls_version']:
                if 'TLSv1.3' not in result['tls_version']:
                    analysis['score'] -= 10
                    analysis['issues'].append(
                        f'Server {result["server"]} not using TLS 1.3'
                    )
            
            if result['issues']:
                analysis['score'] -= 10
                analysis['issues'].extend(
                    [f'Server {result["server"]}: {issue}'
                     for issue in result['issues']]
                )

        return analysis

    def analyze_additional_security(self):
        """Analyze additional email security measures"""
        # Check for DANE (DNS-Based Authentication of Named Entities)
        try:
            tlsa_records = self.query_record(
                'TLSA',
                f'_25._tcp.{self.domain}'
            )
            self.results['dane'] = {
                'present': bool(tlsa_records),
                'records': tlsa_records
            }
        except:
            self.results['dane'] = {
                'present': False,
                'records': []
            }

        # Check for MTA-STS (SMTP MTA Strict Transport Security)
        try:
            mta_sts_records = self.query_record(
                'TXT',
                f'_mta-sts.{self.domain}'
            )
            self.results['mta_sts'] = {
                'present': any(
                    'v=STSv1' in r
                    for r in mta_sts_records
                ),
                'records': mta_sts_records
            }
        except:
            self.results['mta_sts'] = {
                'present': False,
                'records': []
            }

        # Check for BIMI (Brand Indicators for Message Identification)
        try:
            bimi_records = self.query_record(
                'TXT',
                f'default._bimi.{self.domain}'
            )
            self.results['bimi'] = {
                'present': any(
                    'v=BIMI1' in r
                    for r in bimi_records
                ),
                'records': bimi_records
            }
        except:
            self.results['bimi'] = {
                'present': False,
                'records': []
            }

    def calculate_overall_score(self):
        """Calculate overall email security score"""
        scores = []
        
        # SPF Score
        if 'spf' in self.results:
            scores.append(
                self.results['spf']['analysis']['score']
                if self.results['spf']['present']
                else 0
            )

        # DMARC Score
        if 'dmarc' in self.results:
            scores.append(
                self.results['dmarc']['analysis']['score']
                if self.results['dmarc']['present']
                else 0
            )

        # MX Score
        if 'mx' in self.results:
            scores.append(
                self.results['mx']['analysis']['score']
                if self.results['mx']['present']
                else 0
            )

        # SMTP Security Score
        if 'smtp' in self.results:
            scores.append(self.results['smtp']['analysis']['score'])

        # Additional Security Measures
        additional_score = 70
        if self.results.get('dane', {}).get('present'):
            additional_score += 10
        if self.results.get('mta_sts', {}).get('present'):
            additional_score += 10
        if self.results.get('bimi', {}).get('present'):
            additional_score += 10
        scores.append(additional_score)

        # Calculate final score
        return sum(scores) / len(scores) if scores else 0

    def save_results(self, output_file):
        """Save analysis results to file"""
        self.results['overall_score'] = self.calculate_overall_score()
        
        scan_results = {
            'timestamp': datetime.now().isoformat(),
            'domain': self.domain,
            'mode': self.mode,
            'results': dict(self.results)
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(scan_results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        """Print analysis summary"""
        overall_score = self.calculate_overall_score()
        
        self.logger.info(f"\nEmail Security Analysis Summary for {self.domain}")
        self.logger.info(f"Overall Security Score: {overall_score:.1f}/100")

        # SPF Summary
        if 'spf' in self.results:
            spf = self.results['spf']
            self.logger.info("\nSPF Configuration:")
            if spf['present']:
                self.logger.info(f"Record: {spf['record']}")
                self.logger.info(f"Score: {spf['analysis']['score']}/100")
                if spf['analysis']['issues']:
                    self.logger.info("Issues:")
                    for issue in spf['analysis']['issues']:
                        self.logger.info(f"- {issue}")
            else:
                self.logger.info("Not configured")

        # DMARC Summary
        if 'dmarc' in self.results:
            dmarc = self.results['dmarc']
            self.logger.info("\nDMARC Configuration:")
            if dmarc['present']:
                self.logger.info(f"Record: {dmarc['record']}")
                self.logger.info(f"Score: {dmarc['analysis']['score']}/100")
                if dmarc['analysis']['issues']:
                    self.logger.info("Issues:")
                    for issue in dmarc['analysis']['issues']:
                        self.logger.info(f"- {issue}")
            else:
                self.logger.info("Not configured")

        # MX Summary
        if 'mx' in self.results:
            mx = self.results['mx']
            self.logger.info("\nMX Configuration:")
            if mx['present']:
                self.logger.info(f"Score: {mx['analysis']['score']}/100")
                self.logger.info("Servers:")
                for record in mx['records']:
                    self.logger.info(
                        f"- {record['server']} (Priority: {record['priority']})"
                    )
                if mx['analysis']['issues']:
                    self.logger.info("Issues:")
                    for issue in mx['analysis']['issues']:
                        self.logger.info(f"- {issue}")
            else:
                self.logger.info("Not configured")

        # SMTP Security Summary
        if 'smtp' in self.results:
            smtp = self.results['smtp']
            self.logger.info("\nSMTP Security:")
            self.logger.info(f"Score: {smtp['analysis']['score']}/100")
            if smtp['analysis']['issues']:
                self.logger.info("Issues:")
                for issue in smtp['analysis']['issues']:
                    self.logger.info(f"- {issue}")

        # Additional Security Measures
        self.logger.info("\nAdditional Security Measures:")
        self.logger.info(
            f"DANE: {'Configured' if self.results.get('dane', {}).get('present') else 'Not configured'}"
        )
        self.logger.info(
            f"MTA-STS: {'Configured' if self.results.get('mta_sts', {}).get('present') else 'Not configured'}"
        )
        self.logger.info(
            f"BIMI: {'Configured' if self.results.get('bimi', {}).get('present') else 'Not configured'}"
        )

    def start_analysis(self, output_file=None):
        """Start email security analysis"""
        self.logger.info(f"Starting email security analysis for {self.domain}")
        self.logger.info(f"Analysis mode: {self.mode}")
        
        try:
            # Basic analysis
            self.analyze_spf()
            self.analyze_dmarc()
            self.analyze_mx()
            
            # Full analysis
            if self.mode == 'full':
                self.analyze_smtp_security()
                self.analyze_additional_security()
                
        except KeyboardInterrupt:
            self.logger.info("Analysis interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during analysis: {str(e)}")
        finally:
            if output_file:
                self.save_results(output_file)
            self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Email Security Analyzer")
    parser.add_argument("domain", help="Target domain to analyze")
    parser.add_argument("--mode", choices=['basic', 'full'],
                      default='full', help="Analysis mode")
    parser.add_argument("--output", help="Output file for analysis results")
    args = parser.parse_args()

    analyzer = EmailSecurityAnalyzer(args.domain, args.mode)
    analyzer.start_analysis(args.output)
